package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.forbiddenItems;

import eu.mhsl.craftattack.core.appliance.Appliance;
import org.bukkit.Material;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Villager;
import org.bukkit.event.Listener;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Set;
import java.util.function.Function;

public class ForbiddenItems extends Appliance {
    Set<Function<Material, Boolean>> forbiddenMaterials = Set.of(
        (material) -> material.name().startsWith("NETHERITE"),
        (material) -> material.equals(Material.TOTEM_OF_UNDYING),
        (material) -> material.equals(Material.END_CRYSTAL),
        (material) -> material.equals(Material.MACE),
        (material) -> material.equals(Material.ENCHANTED_GOLDEN_APPLE)
    );

    Set<Function<Entity, Boolean>> forbiddenEntities = Set.of(
        (entity) -> {
            if(!(entity instanceof Villager villager)) return false;
            return List.of(
                Villager.Profession.LIBRARIAN,
                Villager.Profession.ARMORER
            ).contains(villager.getProfession());
        }
    );

    Set<Function<PotionEffect, Boolean>> forbiddenPotions = Set.of(
        (potion) -> !potion.getType().equals(PotionEffectType.INSTANT_HEALTH) && potion.getAmplifier() >= 2
    );

    public boolean isForbidden(Material material) {
        return this.forbiddenMaterials.stream()
            .anyMatch(test -> test.apply(material));
    }

    public boolean isForbidden(Entity entity) {
        return this.forbiddenEntities.stream()
            .anyMatch(test -> test.apply(entity));
    }

    public boolean isForbidden(PotionEffect potionEffect) {
        return this.forbiddenPotions.stream()
            .anyMatch(test -> test.apply(potionEffect));
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(
            new ForbiddenMaterialListener()
        );
    }
}
